from typing import Callable, Iterable, List, Optional, Union
from datetime import datetime, timedelta, timezone

from pydantic import validator
from pydantic import BaseModel, Field


def _parse_date(timestamp: str) -> datetime:
    epoch_start = datetime(1601, 1, 1)
    delta = timedelta(microseconds=int(timestamp))
    return (epoch_start + delta).replace(tzinfo=timezone.utc).astimezone()


class MetaInfo:
    last_visited_desktop: str
    last_visited: Optional[str] = None


class Bookmark(BaseModel):
    date_added: datetime
    date_modified: Optional[datetime]
    guid: str
    id_: str = Field(..., alias="id")
    name: str
    type_: str = Field(..., alias="type")
    url: str

    @validator("type_")
    def check_type(cls, v: str) -> str:
        if v != "url":
            raise ValueError("Not a bookmark with URL")
        return v

    @validator("date_added", pre=True)
    def parse_date_added(cls, v: Optional[str]):
        return _parse_date(v) if v else None

    @validator("date_modified", pre=True)
    def parse_date_modified(cls, v: Optional[str]):
        return _parse_date(v) if v else None


class Folder(BaseModel):
    children: List[Union["Folder", Bookmark]] = Field(default_factory=list)
    date_added: datetime
    date_modified: Optional[datetime]
    guid: str
    id_: str = Field(..., alias="id")
    name: str
    type_: str = Field(..., alias="type")

    @validator("type_")
    def check_type(cls, v: str) -> str:
        if v != "folder":
            raise ValueError("Not a folder")
        return v

    @validator("date_added", pre=True)
    def parse_date_added(cls, v: Optional[str]):
        return _parse_date(v) if v else None

    @validator("date_modified", pre=True)
    def parse_date_modified(cls, v: Optional[str]):
        return _parse_date(v) if v else None


Folder.update_forward_refs()


def iter_bookmarks(item: Union[Folder, Bookmark]) -> Iterable[Bookmark]:
    if isinstance(item, Folder):
        for i in item.children:
            yield from iter_bookmarks(i)
    else:
        yield item


class Roots(BaseModel):
    other: Folder
    synced: Folder
    bookmarks_bar: Folder = Field(..., alias="bookmarks_bar")

    def iter_bookmarks(self) -> Iterable[Bookmark]:
        for category in [self.bookmarks_bar, self.other, self.synced]:
            yield from iter_bookmarks(category)


class BookmarksModel(BaseModel):
    checksum: str
    roots: Roots
    sync_metadata: str
    version: int

    def find_bookmark(self, flt: Callable[[Bookmark], bool]) -> Optional[Bookmark]:
        for bm in self.roots.iter_bookmarks():
            if flt(bm):
                return bm

        return None



from pathlib import Path
from shutil import copy
from typing import Optional

from pydantic.tools import parse_file_as
from .chrome_system_paths import get_chrome_bookmarks_path


from .models import BookmarksModel, Folder, Bookmark


def load_bookmarks(
    path: Optional[Path] = None, create_backup: bool = True
) -> BookmarksModel:
    if path is None:
        path = get_chrome_bookmarks_path()

    if create_backup:
        copy(path, "bookmarks-backup.json")

    return parse_file_as(BookmarksModel, path, encoding="utf-8")


__all__ = ["load_bookmarks", "BookmarksModel", "Folder", "Bookmark"]



import sys
from pathlib import Path


def get_chrome_bookmarks_path() -> Path:
    platform = sys.platform.lower()
    if "linux" in platform:
        platform_specific_path = Path("~/.config/google-chrome/Default/Bookmarks")
    elif "darwin" in platform:
        platform_specific_path = Path(
            "~/Library/Application Support/Google/Chrome/Default/Bookmarks"
        )
    elif "win32" in platform:
        platform_specific_path = Path(
            "~\\AppData\\Local\\Google\\Chrome\\User Data\\Default\\Bookmarks"
        )
    else:
        raise ValueError(
            f"It is unknown where the chrome bookmarks path for {sys.platform} is."
        )

    result = platform_specific_path.expanduser()

    if not result.exists():
        raise ValueError(
            f"Chrome bookmarks path determined to be at '{platform_specific_path}' "
            "according to system platform, but nothing exists at that location. Is Chrome installed?"
        )
    
    return result



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['chrome_bookmarks']

package_data = \
{'': ['*']}

install_requires = \
['pydantic>=1.7.1,<2.0.0']

setup_kwargs = {
    'name': 'chromemarks',
    'version': '1.0.0',
    'description': 'A Python library for accessing the local Google Chrome Bookmarks store.',
    'long_description': '',
    'author': 'JosXa',
    'author_email': 'joscha.goetzer@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/JosXa/chrome-bookmarks',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0.0',
}


setup(**setup_kwargs)
